@@ -3,7 +3,8 @@ from typing import Optional
 import torch
 from torch import Size, Tensor
 
-__all__ = ['uniform', 'normal', 'ring', 'orthogonal', 'ones', 'zeros', 'rescale_']
+__all__ = ['uniform', 'normal', 'ring', 'orthogonal', 'symmetric', 'antisymmetric', 'binary', 'ones', 'zeros',
+           'rescale_']
 
 
 def uniform(size: Size, rho: Optional[float] = None, sigma: Optional[float] = None,
@@ -90,6 +91,65 @@ def orthogonal(size: Size, rho: Optional[float] = None, sigma: Optional[float] =
     return W.data
 
 
+def symmetric(size: Size, rho: Optional[float] = None, sigma: Optional[float] = None,
+              scale: Optional[float] = None) -> Tensor:
+    """
+    Symmetric random tensor
+
+    Can either be rescaled according to spectral radius `rho`, spectral norm `sigma`, or `scale`.
+
+    :param size: Size of tensor
+    :param rho: Spectral radius
+    :param sigma: Spectral norm
+    :param scale: Simple rescaling of the standard random matrix
+    :return: A random tensor
+    """
+    W = torch.empty(size).uniform_(-1, 1)
+    W = (W + W.t()) / 2
+    rescale_(W, rho, sigma, scale)
+    return W.data
+
+
+def antisymmetric(size: Size, rho: Optional[float] = None, sigma: Optional[float] = None,
+                  scale: Optional[float] = None) -> Tensor:
+    """
+    Antisymmetric random tensor
+
+    Can either be rescaled according to spectral radius `rho`, spectral norm `sigma`, or `scale`.
+
+    :param size: Size of tensor
+    :param rho: Spectral radius
+    :param sigma: Spectral norm
+    :param scale: Simple rescaling of the standard random matrix
+    :return: A random tensor
+    """
+    W = torch.empty(size).uniform_(-1, 1)
+    W = (W - W.t()) / 2
+    rescale_(W, rho, sigma, scale)
+    return W.data
+
+
+def binary(size: Size, rho: Optional[float] = None, sigma: Optional[float] = None,
+           scale: Optional[float] = None) -> Tensor:
+    """
+    Binary random tensor (±1)
+
+    Used as input matrix for Ring Reservoirs. See:
+    C. Gallicchio & A. Micheli (2020). Ring Reservoir Neural Networks for Graphs.
+    In 2020 International Joint Conference on Neural Networks (IJCNN), IEEE.
+    https://doi.org/10.1109/IJCNN48605.2020.9206723
+
+    :param size: Size of tensor
+    :param rho: Spectral radius
+    :param sigma: Spectral norm
+    :param scale: Simple rescaling of the standard random matrix
+    :return: A random tensor
+    """
+    W = torch.empty(size).uniform_(-1, 1).sign()
+    rescale_(W, rho, sigma, scale)
+    return W.data
+
+
 def ones(size: Size, rho: Optional[float] = None, sigma: Optional[float] = None,
          scale: Optional[float] = None) -> Tensor:
     """
